/*

	jrtos.c  -  main C file containing processes and main

    Copyright (C) 2007  J. op den Brouw <jesse@opdenbrouw.nl>

	This file is part of jrtos version 0.2

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

/***********************************************************************
 This is a second version of JRTOS, a study on how to build a simple
 multitasking system with Real Time capabilities. In this version,
 simple semaphores are implemented, the time slice is adjustable to
 your needs, tasks (still called processes) can give their slice up
 (called yielding). A first attempt is made to do time book keeping,
 using the timer ticks (not time slice time). It is tested on an
 ATmega16, using an AVR Dragon ICE. It is believed it als works on
 an ATmega32, but this is currently not tested.
 There are no "devices" implemented, such as the ADC and the UART.
 The scedular is still round robin, with no priorities.
***********************************************************************/

#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdint.h>

#include "process.h"
#include "semaphore.h"

/* Used by the tasks, put in here so you can see it in a watch window */
volatile uint32_t teller1=0;
volatile uint32_t teller2=0;
volatile uint32_t teller3=0;
volatile uint8_t dummy = 0;
volatile int32_t crit_count = 0;

semaphore s;
semaphore mutex;

/* The tasks!!!!!!! */
void proc0(void) {
	uint8_t i;

	/* Wait for s */
	down(&s);
	while (1) {
		for (i=0; i<10; i++) {
			teller1++;
		}
		yield();
	}
}
void proc1(void) {
	/* Three times: give up! */
	yield();
	yield();
	yield();
	/* Signal s */
	up(&s);
	while (1) {
		teller2++;
	}
}
void proc2(void) {
	while (1) {
		teller3++;
	}
}

/* this one gets lost */
int proclost(void) {
	return 0;
}

void crit1(void) {
	/* volatile uint16_t waste; */

	/* mutex on variable */
	while (1) {
		down(&mutex);
		/*if (crit_count == 10) {
			crit_count = 0;
		} else {
			crit_count++;
		}*/
		/*if (crit_count<100)*/
		crit_count++;
		up(&mutex);
		/*for (waste=0; waste<1000; waste++);*/
	}
}

void crit2(void) {
	/* volatile uint16_t waste; */

	/* mutex on variable */
	while (1) {
		down(&mutex);
		/*if (crit_count == 10) {
			crit_count = 0;
		} else {
			crit_count++;
		}*/
		/*if (crit_count>-100)*/
			crit_count--;
		up(&mutex);
		/*for (waste=0; waste<1000; waste++);*/
	}
}


/* Main routine. Is sets the context structure so tasks can run. */
int main() {

	/* Clear contexts */
	clear_process_list();

	/* Add the processes */
	/*add_process(proc0);
	add_process(proc1);
	add_process(proc2);
	add_process(proclost);*/
	add_process(crit1);
	add_process(crit2);

	/* Add me to the list */
	/* NOTE: after this routine, local vars cannot be */
	/* addressed anymore !!!!!!!!!!!!!!!!!!!!!!!!!!!! */
	add_process_main();


	/* Initialize semaphore */
	init_sem(&s, 0);
	init_sem(&mutex, 1);

	/* Set up timer for context switches */
	setup_timer();

	/* Free interrupts */
	sei();

	/* Do something */
	while(1) {
		dummy++;
	}

	/* Please, do NEVER return from this task */
	return 0;

}
